from setuptools import setup

setup(
    name='jarbas_hive_mind_red',
    version='0.1',
    packages=['jarbas_hive_mind_red'],
    install_requires=["jarbas_hive_mind>=0.9.1"],
    url='https://github.com/OpenJarbas/HiveMind-NodeRed',
    license='MIT',
    author='jarbasAI',
    author_email='jarbasai@mailfence.com',
    description='Mycroft Node Red'
)
